@@ -33,7 +33,7 @@ use Excel::Writer::XLSX::Package::XMLwriter;
 use Excel::Writer::XLSX::Utility qw(xl_cell_to_rowcol xl_rowcol_to_cell);
 
 our @ISA     = qw(Excel::Writer::XLSX::Package::XMLwriter);
-our $VERSION = '0.59';
+our $VERSION = '0.60';
 
 
 ###############################################################################
@@ -84,6 +84,7 @@ sub new {
     $self->{_custom_colors}      = [];
     $self->{_doc_properties}     = {};
     $self->{_localtime}          = [ localtime() ];
+    $self->{_num_vml_files}      = 0;
     $self->{_num_comment_files}  = 0;
     $self->{_optimization}       = 0;
     $self->{_x_window}           = 240;
@@ -841,8 +842,8 @@ sub _store_workbook {
     # Convert the SST strings data structure.
     $self->_prepare_sst_string_data();
 
-    # Prepare the worksheet cell comments.
-    $self->_prepare_comments();
+    # Prepare the worksheet VML elements such as comments and buttons.
+    $self->_prepare_vml_objects();
 
     # Set the defined names for the worksheets such as Print Titles.
     $self->_prepare_defined_names();
@@ -1484,22 +1485,27 @@ sub _prepare_drawings {
 
 ###############################################################################
 #
-# _prepare_comments()
+# _prepare_vml_objects()
 #
-# Iterate through the worksheets and set up the comment data.
+# Iterate through the worksheets and set up the VML objects.
 #
-sub _prepare_comments {
+sub _prepare_vml_objects {
 
-    my $self         = shift;
-    my $comment_id   = 0;
-    my $vml_data_id  = 1;
-    my $vml_shape_id = 1024;
+    my $self          = shift;
+    my $comment_id    = 0;
+    my $vml_data_id   = 1;
+    my $vml_shape_id  = 1024;
+    my $vml_files     = 0;
+    my $comment_files = 0;
 
     for my $sheet ( @{ $self->{_worksheets} } ) {
 
-        next unless $sheet->{_has_comments};
+        next unless $sheet->{_has_vml};
+        $vml_files++;
+        $comment_files++ if $sheet->{_has_comments};
 
-        my $count = $sheet->_prepare_comments( $vml_data_id, $vml_shape_id,
+
+        my $count = $sheet->_prepare_vml_objects( $vml_data_id, $vml_shape_id,
             ++$comment_id );
 
         # Each VML file should start with a shape id incremented by 1024.
@@ -1507,10 +1513,11 @@ sub _prepare_comments {
         $vml_shape_id += 1024 * int( ( 1024 + $count ) / 1024 );
     }
 
-    $self->{_num_comment_files} = $comment_id;
+    $self->{_num_vml_files}     = $vml_files;
+    $self->{_num_comment_files} = $comment_files;
 
     # Add a font format for cell comments.
-    if ( $comment_id > 0 ) {
+    if ( $comment_files > 0 ) {
         my $format = Excel::Writer::XLSX::Format->new(
             \$self->{_xf_format_indices},
             \$self->{_dxf_format_indices},
@@ -2047,9 +2054,11 @@ sub _write_workbook_pr {
     my $show_ink_annotation    = 0;
     my $auto_compress_pictures = 0;
     my $default_theme_version  = 124226;
+    my $codename               = $self->{_vba_codename};
     my @attributes;
 
-    push @attributes, ( 'date1904' => 1 ) if $date_1904;
+    push @attributes, ( 'codeName' => $codename ) if $codename;
+    push @attributes, ( 'date1904' => 1 )         if $date_1904;
     push @attributes, ( 'defaultThemeVersion' => $default_theme_version );
 
     $self->xml_empty_tag( 'workbookPr', @attributes );
